# (c) Copyright 2003-2008. CodeWeavers, Inc.
package CXLog;
use strict;

# Define the module interface
use vars qw(@ISA @EXPORT);
use Exporter ();
@ISA    = "Exporter";
@EXPORT = qw(cxlog cxname0 cxwarn cxerr);


my $cx_log;
my $fh;
local *OLD_STDERR;
my $channels;

sub open($$)
{
    return undef if (defined $cx_log);
    $cx_log=$_[0];
    return undef if (!defined $cx_log);

    my $mode=$_[1] || ">>";
    if (CORE::open($fh, $mode, $cx_log))
    {
        my $tmp=select $fh; $| = 1; select $tmp; # Make unbuffered;
    }
    else
    {
        $cx_log=undef;
        $fh=undef;
    }
    return $fh;
}

sub fdopen($)
{
    return undef if (defined $cx_log);

    my ($fd)=@_;
    $cx_log="&=$fd";
    CORE::open($fh, ">>&=$fd");
    return $fh;
}

sub close()
{
    if ($fh)
    {
        print $fh "Logging turned off\n";
        close($fh);
        $cx_log=undef;
        $fh=undef;
    }
    if (defined fileno OLD_STDERR)
    {
        CORE::open STDERR, ">&OLD_STDERR";
        CORE::close OLD_STDERR;
        my $tmp=select STDERR; $| = 1; select $tmp; # Make unbuffered
    }
}

sub set_channel_default($$)
{
    my ($channel, $default)=@_;
    $channels->{$channel}=$default if (!defined $channels->{$channel});
}

sub is_channel_on($)
{
    my ($channel)=@_;
    return ($fh and $channels->{$channel});
}

sub get_filename()
{
    return $cx_log;
}

sub is_on()
{
    return $fh;
}

sub cxlog(@)
{
    print $fh @_ if ($fh);
}

BEGIN {
    if (!defined $ENV{CX_LOG})
    {
        # Nothing to do then
        ;
    }
    elsif ($ENV{CX_LOG} eq "-")
    {
        CXLog::fdopen(2);
        print $fh "\n\n";
        print $fh "***** ",scalar localtime(),"\n";
        print $fh "Starting: '$0' '",join("' '",@ARGV),"'\n\n";
    }
    elsif (CXLog::open($ENV{CX_LOG}, ">>") or
           CXLog::open($ENV{CX_LOG}, ">") # Needed for fifos on Solaris
          )
    {
        print $fh "\n\n";
        print $fh "***** ",scalar localtime(),"\n";
        print $fh "Starting: '$0' '",join("' '",@ARGV),"'\n\n";
        if (defined fileno STDERR)
        {
            CORE::open OLD_STDERR, ">&STDERR";
            CORE::open STDERR, ">&=" . fileno($fh);
            my $tmp=select STDERR; $| = 1; select $tmp; # Make unbuffered
        }
    }
    if (defined $ENV{CX_DEBUGMSG})
    {
        foreach my $chan_spec (split /,+/, $ENV{CX_DEBUGMSG})
        {
            $chan_spec =~ s/^([-+])//;
            $channels->{$chan_spec}=(($1 || "+") eq "+" ? 1 : 0);
        }
    }
}


#####
#
# Error and warning reporting interface
#
#####

sub cxname0()
{
    my $name0=$0;
    $name0 =~ s+^.*/++;
    return $name0;
}

sub cxwarn(@)
{
    print STDERR cxname0(), ":warning: ", @_;
}

sub cxerr(@)
{
    print STDERR cxname0(), ":error: ", @_;
}

my $cxtime_hires;
sub cxtime()
{
    local $@;
    if (!defined $cxtime_hires)
    {
        $cxtime_hires=is_on() ? eval { require Time::HiRes } : 0;
    }
    return eval { Time::HiRes::time() } if ($cxtime_hires);
    return time();
}

return 1;
